import 'dart:async';
import 'package:built_collection/built_collection.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter/services.dart';
import 'dart:io';
import 'package:mzanalytics_mybmw/mzanalytics_ios.dart';
import 'package:mzanalytics_mybmw/mzanalytics_android.dart';

class MzanalyticsMybmw {
  ///初始化SDK
  static Future<void> appid(String aid) async {
    try {
      if (Platform.isIOS) {
        MzanalyticsiOS.appid(aid);
      } else if (Platform.isAndroid) {
        MzanalyticsAndroid.initEvent();
      } else if (Platform.isMacOS) {}
    } on PlatformException {}
  }

  ///记录页面
  static Future<void> trackPageView(Map parameter) async {
    try {
      if (Platform.isIOS) {
        MzanalyticsiOS.trackPageView(parameter);
      } else if (Platform.isAndroid) {
        MzanalyticsAndroid.trackPageView(parameter);
      }
    } on PlatformException {}
  }

  ///基础事件
  static Future<void> trackEvent(Map parameter) async {
    try {
      if (Platform.isIOS) {
        MzanalyticsiOS.trackEvent(parameter);
      } else if (Platform.isAndroid) {
        MzanalyticsAndroid.trackEvent(parameter);
      }
    } on PlatformException {}
  }

  ///自定义方法
  static Future<void> trackSiteCustomEvent(Map parameter) async {
    try {
      if (Platform.isIOS) {
        MzanalyticsiOS.trackSiteCustomEvent(parameter);
      } else if (Platform.isAndroid) {
        MzanalyticsAndroid.trackSiteCustomEvent(parameter);
      }
    } on PlatformException {}
  }

  ///心跳事件
  static Future<void> trackPulse() async {
    try {
      if (Platform.isIOS) {
        MzanalyticsiOS.trackPulse();
      } else if (Platform.isAndroid) {
        MzanalyticsAndroid.trackPulse();
      }
    } on PlatformException {}
  }

  ///获取cid
  // ignore: missing_return
  static Future<String> getCid() async {
    try {
      if (Platform.isIOS) {
        final String cid = await MzanalyticsiOS.getCid();
        return cid;
      } else if (Platform.isAndroid) {
        final String cid = await MzanalyticsAndroid.getCid();
        return cid;
      }
    } on PlatformException {}
  }

  ///获取oaid
  // ignore: missing_return
  static Future<String> getOaid() async {
    try {
      if (Platform.isAndroid) {
        final String oaid = await MzanalyticsAndroid.getOaid();
        return oaid;
      }
    } on PlatformException {}
  }

  ///deeplink
  static Future<void> onDeepLinkReceived(String url) async {
    try {
      if (Platform.isIOS) {
        MzanalyticsiOS.handleOpenURL(url);
      } else if (Platform.isAndroid) {
        MzanalyticsAndroid.handleOpenURL(url);
      }
    } on PlatformException {}
  }

  /// 自定义cd和cm统一对外方法 */

  // ignore: missing_return
  static Future<Map> getCdAndCm() async {
    try {
      if (Platform.isAndroid) {
        return MzanalyticsAndroid.getCdAndCm();
      } else if (Platform.isIOS) {
        return MzanalyticsiOS.getCdAndCm();
      }
    } on FileSystemException {
      return null;
    }
  }

  ///设置全局cd
  static Future<void> setCd(String cdKey, String cdValue) async {
    try {
      if (Platform.isAndroid) {
        MzanalyticsAndroid.setCd(cdKey, cdValue);
      } else if (Platform.isIOS) {
        MzanalyticsiOS.setCd(cdKey, cdValue);
      }
    } on FileSystemException {}
  }

  ///设置全局cm
  static Future<void> setCm(String cmKey, String cmValue) async {
    try {
      if (Platform.isAndroid) {
        MzanalyticsAndroid.setCm(cmKey, cmValue);
      } else if (Platform.isIOS) {
        MzanalyticsiOS.setCm(cmKey, cmValue);
      }
    } on FileSystemException {}
  }

  ///删除全局cd
  static Future<void> deleteCd(String cdKey) async {
    try {
      if (Platform.isAndroid) {
        MzanalyticsAndroid.deleteCd(cdKey);
      } else if (Platform.isIOS) {
        MzanalyticsiOS.deleteCd(cdKey);
      }
    } on FileSystemException {}
  }

  ///删除全局cm
  static Future<void> deleteCm(String cmKey) async {
    try {
      if (Platform.isAndroid) {
        MzanalyticsAndroid.deleteCm(cmKey);
      } else if (Platform.isIOS) {
        MzanalyticsiOS.deleteCm(cmKey);
      }
    } on FileSystemException {}
  }

  ///isAutoTrackpage
  // ignore: missing_return
  static Future<bool> isAutoTrackpage() async {
    try {
      if (Platform.isAndroid) {
        return MzanalyticsAndroid.isAutoTrackpage();
      } else if (Platform.isIOS) {
        return MzanalyticsiOS.isAutoTrackpage();
      }
    } on FileSystemException {}
  }

  ///getBlackList
  // ignore: missing_return
  static Future<List> getBlackList() async {
    try {
      if (Platform.isAndroid) {
        return MzanalyticsAndroid.getBlackList();
      } else if (Platform.isIOS) {
        return MzanalyticsiOS.getBlackList();
      }
    } on FileSystemException {
      if (kIsWeb) {}
    }
  }
}

///（可选）导航路由观察者方法
// ignore: camel_case_types
class mz_navigation_history_observer extends NavigatorObserver {
  /// A list of all the past routes
  List<Route<dynamic>> _history = <Route<dynamic>>[];

  /// Gets a clone of the navigation history as an immutable list.
  get history => BuiltList<Route<dynamic>>.from(_history);

  /// Gets the top route in the navigation stack.
  get top => _history.last;

  /// A list of all routes that were popped to reach the current.
  List<Route<dynamic>> _poppedRoutes = <Route<dynamic>>[];

  /// Gets a clone of the popped routes as an immutable list.
  get poppedRoutes => BuiltList<Route<dynamic>>.from(_poppedRoutes);

  /// Gets the next route in the navigation history, which is the most recently popped route.
  get next => _poppedRoutes.last;

  /// A stream that broadcasts whenever the navigation history changes.
  StreamController _historyChangeStreamController =
      StreamController.broadcast();

  /// Accessor to the history change stream.
  get historyChangeStream => _historyChangeStreamController.stream;

  static final mz_navigation_history_observer _singleton =
      mz_navigation_history_observer._internal();
  mz_navigation_history_observer._internal();
  factory mz_navigation_history_observer() {
    return _singleton;
  }

  @override
  void didPop(Route<dynamic> route, Route<dynamic> previousRoute) {
    _poppedRoutes.add(_history.last);
    _history.removeLast();
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.pop,
      newRoute: route,
      oldRoute: previousRoute,
    ));
  }

  ///打开页面
  @override
  void didPush(Route<dynamic> route, Route<dynamic> previousRoute) {
    _history.add(route);
    _poppedRoutes.remove(route);
    try {
      MzanalyticsMybmw.isAutoTrackpage().then((bool isAutoTrackpage) {
        MzanalyticsMybmw.getBlackList().then((List blackpvList) {
          if (route.settings.name != null &&
              route.settings.name != '/' &&
              isAutoTrackpage &&
              !blackpvList.contains(route.settings.name)) {
            print('sending...');
            Map selfparams = new Map();
            MzanalyticsMybmw.getCdAndCm().then((Map value) {
              print("读取的自定义事件:" + value.toString());
              if (value != null) {
                selfparams = value;
              }
              //{'pageTitle': route.settings.name;CustomDimMetrics:字典}; {'pageTitle': route.settings.name;字典}

              selfparams['dt'] = route.settings.name;
              print('准备发送的数据：' + selfparams.toString());
              MzanalyticsMybmw.trackPageView(selfparams);
            });
          }
        });
      });
    } on PlatformException {}
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.push,
      newRoute: route,
      oldRoute: previousRoute,
    ));
  }

  @override
  void didRemove(Route<dynamic> route, Route<dynamic> previousRoute) {
    _history.remove(route);
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.remove,
      newRoute: route,
      oldRoute: previousRoute,
    ));
  }

  @override
  void didReplace({Route<dynamic> newRoute, Route<dynamic> oldRoute}) {
    int oldRouteIndex = _history.indexOf(oldRoute);
    _history.replaceRange(oldRouteIndex, oldRouteIndex + 1, [newRoute]);
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.replace,
      newRoute: newRoute,
      oldRoute: oldRoute,
    ));
  }
}

/// A class that contains all data that needs to be broadcasted through the history change stream.
class HistoryChange {
  HistoryChange({this.action, this.newRoute, this.oldRoute});

  final NavigationStackAction action;
  final Route<dynamic> newRoute;
  final Route<dynamic> oldRoute;
}

enum NavigationStackAction { push, pop, remove, replace }
